@@ -124,6 +124,11 @@ Name generation
 .. autofunction:: generate_unique_names
 .. autofunction:: generate_numbered_unique_names
 .. autofunction:: UniqueNameGenerator
+
+Functions for dealing with (large) auxiliary files
+--------------------------------------------------
+
+.. autofunction:: download_from_web_if_not_present
 """
 
 
@@ -1461,7 +1466,7 @@ def string_histogram(iterable, min_value=None, max_value=None,
             bin_nr = bisect(bin_starts, value)-1
             try:
                 bins[bin_nr] += 1
-            except:
+            except Exception:
                 print(value, bin_nr, bin_starts)
                 raise
 
@@ -1850,10 +1855,9 @@ def generate_unique_names(prefix):
 
 
 def generate_numbered_unique_names(prefix, num=None):
-    orig_num = num
-    num = 0
-    if orig_num is None:
-        yield (num, prefix)
+    if num is None:
+        yield (0, prefix)
+        num = 0
 
     while True:
         name = "%s_%d" % (prefix, num)
@@ -1935,11 +1939,45 @@ class MinRecursionLimit(object):
         sys.setrecursionlimit(new_limit)
 
     def __exit__(self, exc_type, exc_val, exc_tb):
+        # Deep recursion can produce deeply nested data structures
+        # (or long chains of to-be gc'd generators) that cannot be
+        # undergo garbage collection with a lower recursion limit.
+        #
+        # Force a garbage collection while the recursion limit is
+        # still high to avoid this situation.
+        #
+        # See https://gitlab.tiker.net/inducer/sumpy/issues/31 for
+        # context.
+        import gc
+        gc.collect()
+
         sys.setrecursionlimit(self.prev_recursion_limit)
 
 # }}}
 
 
+# {{{ download from web if not present
+
+def download_from_web_if_not_present(url, local_name=None):
+    """
+    .. versionadded:: 2017.5
+    """
+
+    from os.path import basename, exists
+    if local_name is None:
+        local_name = basename(url)
+
+    if not exists(local_name):
+        from six.moves.urllib.request import urlopen
+        with urlopen(url) as inf:
+            contents = inf.read()
+
+            with open(local_name, "wb") as outf:
+                outf.write(contents)
+
+# }}}
+
+
 def _test():
     import doctest
     doctest.testmod()
